

proc Set_GRT_Options {} {
    global Options

    load_Options

   if {![info exists Options(grt.channels)]} {set Options(grt.channels) 32768}
   if {![info exists Options(grt.offset)]} {set Options(grt.offset) 0}
   if {![info exists Options(grt.shift)]} {set Options(grt.shift) 1}
   if {![info exists Options(grt.timestamp)]} {set Options(grt.timestamp) 1}

    update_Options
}

proc Dump_Raw_VME_Data_GRT {} {

   set info ""

   set maxmodule [inform vmecontrol inquire grt4pcount]
   if {[string length $maxmodule] > 2} {set maxmodule 0}

   for {set i 1} {$i <= $maxmodule} {incr i} {
      append info "\ngrt4p#$i\n"

      execute-command 19 "Getting Raw Data for GRT4P#$i" "8 $i" 2
      append data [read-register NetVar.EXEC.Response]

         foreach word $data {
            set word 0x$word
            if {$word == 0xaa5555aa} then {
               append info "\n$word - end of dump\n"
               break
            }
            if {$word == 0x55aaaa55} then {
               append info "\n$word - end of event\n"
               continue
            }
            append info "$word "
         }
   }

   append info "\n"

   return "$info"
}

proc define_GRT_config {} {
    global VMEserver

   set maxmodule [inform vmecontrol inquire grt4pcount]
   if {[string length $maxmodule] > 2} {set maxmodule 0}

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set VMEserver [inform vmecontrol VMEmodule2server grt4p#$i]
      set z [catch {eg "(register :name '/$VMEserver/grt4p#$i.ChA.DPP_CSR'  ?object)" } m]
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to locate register grt4p#$i.ChA.DPP_CSR"
          clear-footer
          error $m
      }
      DefineConfig "4 $i [string range [first $m] 2 end]"
   }
}

proc create_GRT_spectra {} {
    global SpectrumServer
    global Options

#   for the GRP 4 channel module

   set ADCs [list A B C D]

   set maxmodule [inform vmecontrol inquire grt4pcount]
   if {[string length $maxmodule] > 2} {set maxmodule 0}
   set maxadc 4

   for {set i 1} {$i <= $maxmodule} {incr i} {

    set VMEserver [inform vmecontrol VMEmodule2server grt4p#$i]
    
    for {set j 0} {$j < $maxadc} {incr j} {

#    histogram used to display the digital trace
#    need to obtain trace length from the board

      set z [catch {eg "(register :name '/$VMEserver/grt4p#$i.Ch[lindex $ADCs $j].DPP_Reg6'  ?value)" } vl]
      if {$z!=0} then {set vl 512}      ;# use default 
      if {$vl==0} then {set vl 512}     ;# use default

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range [expr $vl - 2] :name 'grt4p#$i.adc$j' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum grt4p#$i.adc$j"
          clear-footer
          error $m
      } else {
          if {$Options(grt.timestamp) == 0} then {set t 1} else {set t 4}
          map_spectrum  "grt4p#$i.adc$j [expr 256 + $j + ($i * 4)] 0 $t 8192"
      }

#    histograms for the computed energy & baseline

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $Options(grt.channels) :name 'grt4p#$i.E[set j]' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum grt4p#$i.E[set j]"
          clear-footer
          error $m
      } else {
          map_spectrum  "grt4p#$i.E[set j] [expr $j + ($i * 4)] 1 $Options(grt.offset) $Options(grt.shift)"         ;#  item 1
      }

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $Options(grt.channels) :name 'grt4p#$i.B[set j]' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum grt4p#$i.B[set j]"
          clear-footer
          error $m
      } else {          
          map_spectrum  "grt4p#$i.B[set j] [expr $j + ($i * 4)] 0 $Options(grt.offset) $Options(grt.shift)"        ;#  item 0
      }
     }
    }

    execute-command 17 "" "0 0 0 100 $Options(grt.timestamp)" 2    ;#  set program options
}

